
/*
    Copyright (c) 2002, 2003 Gregory Montoir

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SDL.h"
#include "sdlsms.h"
#include "filters.h"
#include "saves.h"
#include "shared.h"

#ifdef _arch_dreamcast
#include <kos.h>
#include <arch/gdb.h>
#endif


static t_config cfg;
static t_named_filter filters[] = {
  {"2xsai",      FILTER_2XSAI},
  {"super2xsai", FILTER_SUPER2XSAI},
  {"supereagle", FILTER_SUPEREAGLE},
  {"advmame2x",  FILTER_ADVMAME2X},
  {"tv2x",       FILTER_TV2X},
  {"2x",         FILTER_NORMAL2X},
  {"bilinear",   FILTER_BILINEAR},
  {"dotmatrix",  FILTER_DOTMATRIX}
};

#ifdef _arch_dreamcast
static int filidx = -1;




struct RomData  {
    dirent_t dirent;
    STAILQ_ENTRY(RomData) romlist;
};

STAILQ_HEAD(RomDataList, RomData);
static struct RomDataList romList;
static int files = 0;


// from nxDoom by BlackAura
void dc_drawstring(int x, int y, char *string)
{
	bfont_draw_str(vram_s + ((y + 1) * 24 * 640) + (x * 12), 640, 1, string);
}

// from nxDoom by BlackAura
void dc_cls()
{
	int x, y, ofs;
	for(y = 0; y < 480; y++)
	{
		ofs = (640 * y);
		for(x = 0; x < 640; x++)
			vram_s[ofs + x] = 0;
	}
}

// based on dc_cls() from nxDoom by BlackAura
void dc_cla()
{
	int x, y, ofs;
	for(y = 0; y < 480; y++)
	{
		ofs = (640 * y);
		for(x = 24; x < 48; x++)
			vram_s[ofs + x] = 0;
	}
}

static char **filenames;

static void create_list(const char *dir)    {
    file_t d;
    dirent_t *ent;
    int i = 0;
    struct RomData *data;

    STAILQ_INIT(&romList);

    d = fs_open(dir, O_DIR);

    while((ent = fs_readdir(d)))    {
        // Ignore subdirectories
        if(ent->attr & O_DIR)   {
            continue;
        }

		data = (struct RomData *)malloc(sizeof(struct RomData));
		data->dirent = *ent;

		STAILQ_INSERT_TAIL(&romList, data, romlist);
		++files;
    }

    filenames = (char **)malloc(sizeof(char *) * files);

    STAILQ_FOREACH(data, &romList, romlist) {
        filenames[i] = (char *)malloc(strlen(data->dirent.name) + 1);
        strcpy(filenames[i], data->dirent.name);
        ++i;
    }
}

static void destroy_list()  {
    struct RomData *ent, *ent2;
    int i;

    ent = STAILQ_FIRST(&romList);
    while(ent)  {
        ent2 = STAILQ_NEXT(ent, romlist);
        STAILQ_REMOVE_HEAD(&romList, romlist);
        free(ent);
        ent = ent2;
    }

    for(i = 0; i < files; ++i)  {
        free(filenames[i]);
    }

    free(filenames);
    files = 0;
}

const char *menu(const char *dir)   {
    int topItem = 0;
    int selected = 0;
    int i;
    maple_device_t *cont1;
    cont_state_t *state;
    uint64 timer = timer_ms_gettime64();
	char filename[MAX_FN_LEN];

    cont1 = maple_enum_type(0, MAPLE_FUNC_CONTROLLER);

	create_list(dir);

	if (files == 0)
		dc_drawstring(4, 6, "Error: No ROMs found");

    dc_cls();
				dc_drawstring(2, 1, ">");

    for(;;) {
        if(timer + 200 < timer_ms_gettime64())  {
            state = (cont_state_t *)maple_dev_status(cont1);

            if(state)   {
                if(state->buttons & CONT_DPAD_DOWN) {
                    ++selected;

                    if(selected == files)   {
                        selected = files - 1;
                    }
                    else if(selected == 15) {
                        if(topItem + 15 < files)    {
                            ++topItem;
                        }

                        selected = 14;
                    }
dc_cla();

				dc_drawstring(2, 1+selected, ">");
                    timer = timer_ms_gettime64();
                }
                if(state->buttons & CONT_DPAD_UP)   {
                    --selected;

                    if(selected == -1)  {
                        if(topItem) {
                            --topItem;
                        }

                        selected = 0;
                    }
dc_cla();
				dc_drawstring(2, 1+selected, ">");
                    timer = timer_ms_gettime64();
                }
                if(state->buttons & CONT_A) {
                    sprintf(filename, "%s/%s", dir,
                            filenames[selected + topItem]);
                    break;
                }/*
                if(state->buttons & CONT_B) {
                    return NULL;
                }*/
            }
        }

        for(i = 0; i < 15 && i + topItem < files; ++i)  {
        	dc_drawstring(4, 1 + i, filenames[i + topItem]);
        }
    }

    destroy_list(dir);

    return filename;
}
void dc_rommenu()
{
    file_t d;
    dirent_t *ent;

    d = fs_open("/cd/roms", O_DIR);

    while((ent = fs_readdir(d)))    {
        // Ignore subdirectories
        if(ent->attr & O_DIR)   {
            continue;
        }



    }
		filenames = (char **)malloc(sizeof(char *) * files);

}

void dc_optionmenu()
{
	int csel = 0;
	int lsel = -1;
	int firstPad = 0;
	cont_cond_t cond;

	dc_cls();

	while(1)	{
		firstPad = maple_first_controller();
		cont_get_cond(firstPad, &cond);

		dc_drawstring(4, 1, "SMS+SDL\\DC  0.1");
		if(sms.use_fm == 0)
			dc_drawstring(4, 6, "YM2413 sound: No");
		else
			dc_drawstring(4, 6, "YM2413 sound: Yes");
		if(sms.country == TYPE_OVERSEAS)
			dc_drawstring(4, 7, "Machine type: Overseas");
		else
			dc_drawstring(4, 7, "Machine type: Domestic");
		if(cfg.nosound == 0)
			dc_drawstring(4, 8, "Sound: Yes");
		else
			dc_drawstring(4, 8, "Sound: No");
		if(filidx == -1)
			dc_drawstring(4, 9, "Filter: None");
		else if(filidx == 0)
			dc_drawstring(4, 9, "Filter: 2xsai");
		else if(filidx == 1)
			dc_drawstring(4, 9, "Filter: super2xsai");
		else if(filidx == 2)
			dc_drawstring(4, 9, "Filter: supereagle");
		else if(filidx == 3)
			dc_drawstring(4, 9, "Filter: advmame2x");
		else if(filidx == 4)
			dc_drawstring(4, 9, "Filter: tv2x");
		else if(filidx == 5)
			dc_drawstring(4, 9, "Filter: 2x");
		else if(filidx == 6)
			dc_drawstring(4, 9, "Filter: bilinear");
		else if(filidx == 7)
			dc_drawstring(4, 9, "Filter: dotmatrix");

		if(!(cond.buttons & CONT_DPAD_DOWN))	{
			csel++;
			lsel = -1;

			// Jump to top
			if(csel > 4)
				csel = 0;

			while(!(cond.buttons & CONT_DPAD_DOWN))	{
				cont_get_cond(firstPad, &cond);
			}
		}
		if(!(cond.buttons & CONT_DPAD_UP))	{
			csel--;
			lsel = -1;

			// Jump to bottom
			if(csel < 0)
				csel = 4;

			while(!(cond.buttons & CONT_DPAD_UP)) {
				cont_get_cond(firstPad, &cond);
			}
		}
		if(!(cond.buttons & CONT_DPAD_LEFT))	{
			if(csel == 0)	{
				sms.use_fm = 0;
			}
			else if(csel == 1)	{
				sms.country = TYPE_OVERSEAS;
			}
			else if(csel == 2)	{
				cfg.nosound = 1;
			}
			else if(csel == 3)	{
				if(filidx != -1)
					filidx--;
				cfg.filter = filters[filidx].type;
			}

			while(!(cond.buttons & CONT_DPAD_LEFT)) {
				cont_get_cond(firstPad, &cond);
			}
			lsel = -1;
			dc_cls();
		}
		if(!(cond.buttons & CONT_DPAD_RIGHT))	{
			if(csel == 0)	{
				sms.use_fm = 1;
			}
			else if(csel == 1)	{
				sms.country = TYPE_DOMESTIC;
			}
			else if(csel == 2)	{
				cfg.nosound = 0;
			}
			else if(csel == 3)	{
				if(filidx != 7)
					filidx++;
				cfg.filter = filters[filidx].type;
			}

			while(!(cond.buttons & CONT_DPAD_RIGHT)) {
				cont_get_cond(firstPad, &cond);
			}
			lsel = -1;
			dc_cls();
		}

		if(lsel != csel)	{
			dc_cla();
			dc_drawstring(2, 6 + csel, ">");
		}

		lsel = csel;

		if (!(cond.buttons & CONT_B))	{
			return;
		}
	}
}

void dc_main()
{
	int csel = 0;
	int lsel = -1;
	int firstPad = 0;
	cont_cond_t cond;

	sms.use_fm = 0;
	sms.country = TYPE_OVERSEAS;
	cfg.filter = -1;
	cfg.fullscreen = 1;
	cfg.joystick = 1;

	while(1)	{
		firstPad = maple_first_controller();
		cont_get_cond(firstPad, &cond);

		dc_drawstring(4, 1, "SMS+SDL\\DC  0.1");
		dc_drawstring(4, 6, "Select ROMs");
		dc_drawstring(4, 7, "Options");
		dc_drawstring(4, 8, "Exit");

		if(!(cond.buttons & CONT_DPAD_DOWN))	{
			csel++;
			lsel = -1;

			if(csel > 2)
				csel = 0;

			while(!(cond.buttons & CONT_DPAD_DOWN))	{
				cont_get_cond(firstPad, &cond);
			}
		}
		if(!(cond.buttons & CONT_DPAD_UP))	{
			csel--;
			lsel = -1;

			if(csel < 0)
				csel = 2;

			while(!(cond.buttons & CONT_DPAD_UP)) {
				cont_get_cond(firstPad, &cond);
			}
		}

		if(lsel != csel)	{
			dc_cla();
			dc_drawstring(2, 6 + csel, ">");
		}

		lsel = csel;

		if (!(cond.buttons & CONT_A))	{
			switch(csel)	{
				case 0:
					//dc_rommenu();
					dc_cls();
					strcpy(cfg.game_name, menu("/cd/roms"));
					dc_cls();
					//if(cfg.game_name != NULL)
						return;
					break;
				case 1:
					dc_optionmenu();
					dc_cls();
				    csel = 0;
				    lsel = -1;
					break;
				case 2:
					arch_return();
					break;
			}
		}
	}
}
#endif

#ifndef _arch_dreamcast
static int parse_args(int argc, char **argv)
{
  int i;

  /* default virtual console emulation settings */
  sms.use_fm = 0;
  sms.country = TYPE_OVERSEAS;

  cfg.filter = -1;
  //strcpy(cfg.game_name, argv[1]);

  for(i = 2; i < argc; ++i) {
    if(strcasecmp(argv[i], "--fm") == 0)
      sms.use_fm = 1;
    else if(strcasecmp(argv[i], "--japan") == 0)
      sms.country = TYPE_DOMESTIC;
    else if(strcasecmp(argv[i], "--usesram") == 0) {
      cfg.usesram = 1;
      sms.save = 1;
    }
    else if(strcasecmp(argv[i], "--fskip") == 0) {
      if(++i<argc) {
        cfg.frameskip = atoi(argv[i]);
      }
    }
    else if(strcasecmp(argv[i], "--fullspeed") == 0)
      cfg.fullspeed = 1;
    else if(strcasecmp(argv[i], "--fullscreen") == 0)
      cfg.fullscreen = 1;
    else if(strcasecmp(argv[i], "--nosound") == 0)
      cfg.nosound = 1;
    else if(strcasecmp(argv[i], "--joystick") == 0)
      cfg.joystick = 1;
    else if(strcasecmp(argv[i], "--filter") == 0) {
      i++;
      if(i < argc) {
        int j = 0;
        for( ; j < sizeof(filters) / sizeof(filters[0]); ++j) {
          if(strcasecmp(filters[j].name, argv[i]) == 0) {
            cfg.filter = filters[j].type;
            break;
          }
        }
      }
    }
    else
      printf("WARNING: unknown option '%s'.\n", argv[i]);
  }
  return 1;
}
#endif

int main(int argc, char **argv)
{
  printf("%s (Build date %s)\n", SMSSDL_TITLE, __DATE__);
  printf("(C) Charles Mac Donald in 1998, 1999, 2000\n");
  printf("SDL Version by Gregory Montoir (cyx@frenchkiss.net)\n");
  printf("\n");

#ifndef _arch_dreamcast
  if(argc < 2) {
    int i;
    printf("Usage: %s <filename.<SMS|GG>> [--options]\n", argv[0]);
    printf("Options:\n");
    printf(" --fm           \t enable YM2413 sound.\n");
    printf(" --japan        \t set the machine type as DOMESTIC instead of OVERSEAS.\n");
    printf(" --usesram      \t load/save SRAM contents before starting/exiting.\n");
    printf(" --fskip <n>    \t specify the number of frames to skip.\n");
    printf(" --fullspeed    \t do not limit to 60 frames per second.\n");
    printf(" --fullscreen   \t start in fullscreen mode.\n");
    printf(" --joystick     \t use joystick.\n");
    printf(" --nosound      \t disable sound.\n");
    printf(" --filter <mode>\t render using a filter: ");
    for(i = 0; i < sizeof(filters) / sizeof(filters[0]) - 1; ++i)
      printf("%s,", filters[i].name);
    printf("%s.", filters[i].name);
    return 1;
  }
#endif

  memset(&cfg, 0, sizeof(cfg));
#ifndef _arch_dreamcast
  if(!parse_args(argc, argv))
	  return 0;
#endif

  dc_main();

  //gdb_init();

  if(sdlsms_init(&cfg)) {
    sdlsms_emulate();
    sdlsms_shutdown();
  }

  return 0;
}
